import { getCurrentPages, debounce } from "@/utils/index";
import Taro from "@tarojs/taro";

class BaseRouter {
  // 防止重复点击跳转页面
  toPageFlag = false;
  data = null;

  verifyIsLogin(options) {
    const token = Taro.getStorageSync("token");
    if (options?.isLogin && !token) {
      Taro.showToast({ title: "登录后可使用所有功能", icon: "error" });
      return false;
    }
    return true;
  }

  /**
   *
   * @param {*} url 跳转url
   * @param {*} options 要传递的参数
   * @returns
   */
  navigate(url, options) {
    return new Promise(async (resolve) => {
      if (!this.verifyIsLogin(options)) return;
      this.data = options;
      if (this.toPageFlag) return;
      this.toPageFlag = true;
      Taro.navigateTo({
        url,
        events: {
          // 为指定事件添加一个监听器，获取被打开页面传送到当前页面的数据
          acceptDataFromOpenedPage: (data) => {
            resolve(data);
          },
        },
      }).finally(() => {
        setTimeout(() => {
          this.toPageFlag = false;
        }, 1000);
      });
    });
  }

  /**
   *
   * @param {*} url 跳转url
   * @param {*} options 要传递的参数
   * @returns
   */
  reLaunch(url, options) {
    this.data = options;
    return new Promise(async (resolve) => {
      if (!this.verifyIsLogin(options)) return;
      Taro.reLaunch({
        url,
        events: {
          // 为指定事件添加一个监听器，获取被打开页面传送到当前页面的数据
          acceptDataFromOpenedPage: (data) => {
            resolve(data);
          },
        },
      });
    });
  }

  /**
   * 返回上一页面
   * @param delta
   * @param data
   */
  back = debounce((data, delta = 1) => {
    this.data = null;
    const eventChannel = getCurrentPages();
    if (data != null) eventChannel?.emit("acceptDataFromOpenedPage", data);
    Taro.navigateBack({ delta });
  });

  /**
   *
   * @returns 获取上一个页面传递过来的参数
   */
  getData(fn) {
    this.eventChannel = getCurrentPages();
    this.eventChannel.on &&
      this.eventChannel.on("acceptDataFromOpenedPage", (res) => {
        fn && fn(res);
      });
  }
}
export default new BaseRouter();
