import { View } from '@tarojs/components'

import './index.less'
import { useEffect, useState } from 'react'
import Taro from '@tarojs/taro'
import { Popup } from "@taroify/core"

export default function Index({ open, onChange }) {
  let [data, setData] = useState({
    boundaryData: [], // 元素边界信息
    gradient: '',
    hex: '#ff0000',
    rgba: {
      r: 255,
      g: 0,
      b: 0,
      a: 1
    },
    hsb: {
      h: 0,
      s: 100,
      b: 100
    },
    bgcolor: {
      r: 255,
      g: 0,
      b: 0,
      a: 1
    },
    point: [{
      top: 0, // 颜色盒子里位置信息
      left: 0
    }, {
      left: 0 // 颜色条
    }, {
      left: 0 // 透明度
    }],
  })

  useEffect(() => {
    if (open) {
      setTimeout(() => {
        getBoundaryData();
      }, 200)
    }
  }, [open])

  // 获取元素边界值
  const getBoundaryData = () => {
    Taro.createSelectorQuery().selectAll('.range-box').boundingClientRect((res) => {
      if (!res || res.length === 0) {
        setTimeout(() => getBoundaryData(), 20)
        return
      }
      data.boundaryData = res
      setData({ ...data })
    }).exec()
  }

  const confirm = () => {
    onChange({ rgba: `${data.rgba.r}, ${data.rgba.g}, ${data.rgba.b}`, hex: data.hex })
  }

  const close = () => {
    onChange()
  }

  const touchstart = (e) => {
    const index = e.currentTarget.dataset.index;
    const {
      pageX,
      pageY
    } = e.touches[0];
    setPointAndColor(pageX, pageY, index);
  }

  const touchmove = (e) => {
    const index = e.currentTarget.dataset.index;
    const {
      pageX,
      pageY
    } = e.touches[0];
    setPointAndColor(pageX, pageY, index);
  }

  // 设置位置和颜色
  const setPointAndColor = (x, y, index) => {
    const {
      top,
      left,
      width,
      height
    } = data.boundaryData[index];
    // 颜色盒
    if (index == 0) {
      changeColorBox(x, y, index, top, left, width, height);
    }
    // 颜色条
    else if (index == 1) {
      changeColorHSB(x, y, index, top, left, width, height);
    }
    // 透明度
    else if (index == 2) {
      changeTransparency(x, y, index, top, left, width, height);
    }
    // rgb条
    else {
      changeColorRGB(x, y, index, top, left, width, height);
    }
  }

  // 点击操作颜色盒
  const changeColorBox = (x, y, index, top, left, width, height) => {
    let pointTop = Math.max(0, Math.min(parseInt(y - top), height));
    let pointLeft = Math.max(0, Math.min(parseInt(x - left), width));
    // 设置颜色
    let hsb = {
      h: data.hsb.h,
      s: parseInt((100 * pointLeft) / width),
      b: parseInt(100 - (100 * pointTop) / height)
    }
    const rgb = hsbToRgb(hsb);
    let rgba = {
      r: rgb.r,
      g: rgb.g,
      b: rgb.b,
      a: data.rgba.a
    }
    // 设置data
    setData((state) => {
      state.point[index] = {
        top: pointTop,
        left: pointLeft
      }
      state.hsb = hsb
      state.rgba = rgba
      state.hex = '#' + rgbToHex(rgb)
      return { ...state }
    })

  }

  // 颜色条，饱和度
  const changeColorHSB = (x, y, index, top, left, width, height) => {
    let pointLeft = Math.max(0, Math.min(parseInt(x - left), width));
    let hsb = {
      h: parseInt((360 * pointLeft) / width),
      s: 100,
      b: 100,
    }
    let rgb = hsbToRgb(hsb);
    let rgba = {
      r: rgb.r,
      g: rgb.g,
      b: rgb.b,
      a: data.rgba.a
    }
    setData((state) => {
      state.point[index].left = pointLeft
      state.hsb = hsb
      state.rgba = rgba
      state.hex = '#' + rgbToHex(rgb)
      state.bgcolor = hsbToRgb(hsb)
      return { ...state }
    })

  }

  // 透明度
  const changeTransparency = (x, y, index, top, left, width, height) => {
    let pointLeft = Math.max(0, Math.min(parseInt(x - left), width));
    setData((state) => {
      state.point[index].left = pointLeft
      state.rgba.a = (pointLeft / width).toFixed(1)
      return { ...state }
    })
  }

  // rgb条
  const changeColorRGB = (x, y, index, top, left, width, height) => {
    let pointLeft = Math.max(0, Math.min(parseInt(x - left), width));
    let pointValue = parseInt(pointLeft / width * 255);
    let rgba = {
      r: index == 3 ? pointValue : data.rgba.r,
      g: index == 4 ? pointValue : data.rgba.g,
      b: index == 5 ? pointValue : data.rgba.b,
      a: data.rgba.a
    }
    let hsb = rgbToHsb(rgba);
    // 计算颜色条的位置
    let hsb_left = parseInt((hsb.h * data.boundaryData[1].width) / 360);
    // 设置data
    setData((state) => {
      state.point[index].left = pointLeft
      state.point[1].left = hsb_left
      state.hsb = hsb
      state.rgba = rgba
      state.hex = '#' + rgbToHex(rgba)
      state.bgcolor = hsbToRgb(hsb)
      return { ...state }
    })

  }

  // 常用颜色选择
  const selectColor = (event) => {
    setColorByOption(event.currentTarget.dataset.color)
  }

  const setColorByOption = (optionColor) => {
    const {
      r,
      g,
      b,
      a
    } = optionColor;
    let rgba = {
      r: r ? parseInt(r) : 0,
      g: g ? parseInt(g) : 0,
      b: b ? parseInt(b) : 0,
      a: a ? a : 0,
    };
    let hsb = rgbToHsb(rgba);

    setData((state) => {
      state.rgba = rgba
      state.hsb = hsb
      state.hex = '#' + rgbToHex(rgba)
      state.bgcolor = hsbToRgb(hsb)
      return { ...state }
    })

    changePoint();
  }

  const changePoint = () => {
    const [a, b, c,] = data.boundaryData;
    let point = [{
      top: parseInt((100 - data.hsb.b) * a.height / 100), // 颜色盒子
      left: parseInt(data.hsb.s * a.width / 100)
    }, {
      left: data.hsb.h / 360 * b.width // 颜色条
    }, {
      left: data.rgba.a * c.width // 透明度
    }
    ]
    setData((state) => {
      state.point = point
      return { ...state }
    })
  }

  // rgb 转 hex
  const rgbToHex = (rgb) => {
    let hex = [rgb.r.toString(16), rgb.g.toString(16), rgb.b.toString(16)];
    hex.map(function (str, i) {
      if (str.length == 1) {
        hex[i] = '0' + str;
      }
    });
    return hex.join('');
  }

  // rgb 转 hsb
  const rgbToHsb = (rgb) => {
    let hsb = {
      h: 0,
      s: 0,
      b: 0
    };
    let min = Math.min(rgb.r, rgb.g, rgb.b);
    let max = Math.max(rgb.r, rgb.g, rgb.b);
    let delta = max - min;
    hsb.b = max;
    hsb.s = max != 0 ? 255 * delta / max : 0;
    if (hsb.s != 0) {
      if (rgb.r == max) hsb.h = (rgb.g - rgb.b) / delta;
      else if (rgb.g == max) hsb.h = 2 + (rgb.b - rgb.r) / delta;
      else hsb.h = 4 + (rgb.r - rgb.g) / delta;
    } else hsb.h = -1;
    hsb.h *= 60;
    if (hsb.h < 0) hsb.h = 0;
    hsb.s *= 100 / 255;
    hsb.b *= 100 / 255;
    return hsb;
  }

  // hsb 转 rgb 颜色模式  H(hues)表示色相，S(saturation)表示饱和度，B（brightness）表示亮度
  const hsbToRgb = (hsb) => {
    let rgb = {};
    let h = Math.round(hsb.h);
    let s = Math.round((hsb.s * 255) / 100);
    let v = Math.round((hsb.b * 255) / 100);
    if (s == 0) {
      rgb.r = rgb.g = rgb.b = v;
    } else {
      let t1 = v;
      let t2 = ((255 - s) * v) / 255;
      let t3 = ((t1 - t2) * (h % 60)) / 60;
      if (h == 360) h = 0;
      if (h < 60) {
        rgb.r = t1;
        rgb.b = t2;
        rgb.g = t2 + t3;
      } else if (h < 120) {
        rgb.g = t1;
        rgb.b = t2;
        rgb.r = t1 - t3;
      } else if (h < 180) {
        rgb.g = t1;
        rgb.r = t2;
        rgb.b = t2 + t3;
      } else if (h < 240) {
        rgb.b = t1;
        rgb.r = t2;
        rgb.g = t1 - t3;
      } else if (h < 300) {
        rgb.b = t1;
        rgb.g = t2;
        rgb.r = t2 + t3;
      } else if (h < 360) {
        rgb.r = t1;
        rgb.g = t2;
        rgb.b = t1 - t3;
      } else {
        rgb.r = 0;
        rgb.g = 0;
        rgb.b = 0;
      }
    }
    return {
      r: Math.round(rgb.r),
      g: Math.round(rgb.g),
      b: Math.round(rgb.b)
    };
  }
  const { bgcolor, point, rgba } = data
  useEffect(() => {
  }, [])
  return (
    <Popup open={open} rounded placement="bottom"  >
      <View className="cp-box">
        <View className="cp-header">
          <View className="cp-header-icon cancel" onClick={close}></View>
          <View className="cp-header-title" >颜色选取</View>
          <View className="cp-header-icon confirm" onClick={confirm}></View>
        </View>

        <View className="cp-color-box" style={'background:' + ('rgb(' + bgcolor.r + ',' + bgcolor.g + ',' + bgcolor.b + ')') + ';'}>
          <View className="cp-background range-box" data-index="0" onTouchStart={touchstart} onTouchMove={touchmove} >
            <View className="cp-color-mask"></View>
            <View className="cp-pointer" style={'top:' + (point[0].top - 8 + 'px') + ';' + ('left:' + (point[0].left - 8 + 'px') + ';')}></View>
          </View>
        </View>

        <View className="cp-control-box">
          <View className="cp-control-color">
            <View className="cp-control-color-content" style={'background:' + ('rgba(' + rgba.r + ',' + rgba.g + ',' + rgba.b + ',' + rgba.a + ')') + ';'}>{data.hex}</View>
          </View>
          <View className="cp-control-box-item">
            <View className="cp-controller range-box" data-index="1" onTouchStart={touchstart} onTouchMove={touchmove}>
              <View className="cp-hue">
                <View className="cp-circle" style={'left:' + (point[1].left - 12 + 'px') + ';'}></View>
              </View>
            </View>
          </View>
        </View>
      </View>
    </Popup>


  )
}
