import { View } from '@tarojs/components'
import './index.less'
import Taro from '@tarojs/taro'
import { useState, useEffect } from 'react'
import router from '@/baseRouter/index'
import BLESDK from '@/utils/ble'

export default function Index() {
  let [deviceList, setDeviceList] = useState([])
  const connect = (item) => {
    Taro.showLoading({
      title: '正在连接设备',
      mask: true,
    })
    BLESDK.deviceInfo = item
    BLESDK.createBleConnection()
  }

  useEffect(() => {
    BLESDK.startBluetoothDevicesDiscovery()
    Taro.showLoading({ title: '设备搜索中..', mask: true })
    BLESDK.deviceCallBack(deviceCallBack)

    setTimeout(() => {
      Taro.hideLoading()
    }, 3000)
    return () => {
      BLESDK.stopBluetoothDevicesDiscovery()
    }
  }, [])

  const deviceCallBack = (res) => {
    Taro.hideLoading()
    router.back()
  }

  BLESDK.onBluetoothDeviceFound((item) => {
    if (JSON.stringify(deviceList).indexOf(item.mac) != -1) return
    deviceList.push(item)
    setDeviceList([...deviceList])
  })

  return (
    <View className="index deviceList">
      <View className="head">
        <View className="radar">
          <View className="yuan1 yuan"></View>
          <View className="yuan2 yuan"></View>
          <View className="yuan3 yuan"></View>
        </View>
        <View className="text">请确保手机蓝牙开启，同时待激活设备也处于上电开启状态，尽量靠近设备，强信号有助于提高设备激活的成功率!</View>
      </View>
      <View className='list'>
        {deviceList.map(item => <View className='item'>
          <View className='item-img'></View>
          <View className='item-content'>
            <View className='item-content-name'>{item.name}</View>
            <View className='item-content-mac'>{item.mac}</View>
          </View>
          <View className='item-btn' onClick={() => connect(item)}>连接</View>
        </View>)}
      </View>
    </View >
  )
}
