import { View } from '@tarojs/components'
import './index.less'
import { Form, Toast, Cell, Slider, Radio, Switch, Stepper, Button } from "@taroify/core"
import { useCallback, useState } from 'react'
import { hex, strInsert } from '@/utils/sendOrder'
import { debounce } from '@/utils/index'
import BLESDK from '@/utils/ble'

export default function Index() {
  const [ruleForm, setRuleForm] = useState({
    volume: 0, //话筒音量
    music: 0, //音乐音量
    source: 0, //音源设置
    bleSwitch: 0, //蓝牙开关
    soundCadSwitch: 0, //声卡开关
    playMode: 0, //播放模式
    musicControl: 0, //音乐控制
    girth: 0, //超低音强度
    reverberationMode: 0, //混响模式
    microphoneReverberation: 0, //话筒混响
    AI: 0, //AI
  })

  const sendCode = useCallback(debounce((e) => {
    BLESDK.writeBleValue(new Uint8Array(strInsert(e)).buffer)
  }), [])
  const handleChange = (e, key) => {
    setRuleForm({
      ...ruleForm,
      [key]: e
    })
    let str = ''
    switch (key) {
      case 'volume':
        str = `7B05EA05B2${hex(e)}`
        break;
      case 'music':
        str = `7B05EA06B2${hex(e)}`
        break;
      case 'source':
        str = `7B05EA07B2${hex(e)}`
        break;
      case 'bleSwitch':
        str = `7B05EA08B2${hex(e)}`
        break;
      case 'soundCadSwitch':
        str = `7B05EA09B2${hex(e)}`
        break;
      case 'playMode':
        str = `7B05EA0AB2${hex(e)}`
        break;
      case 'musicControl':
        str = `7B05EA0BB2${hex(e)}`
        break;
      case 'girth':
        str = `7B05EA0CB2${hex(e)}`
        break;
      case 'reverberationMode':
        str = `7B05EA0DB2${hex(e)}`
        break;
      case 'microphoneReverberation':
        str = `7B05EA0EB2${hex(e)}`
        break;
      case 'AI':
        str = `7B05EA11B2${hex(e)}`
        break;
    }
    sendCode(str)
  }

  return (
    <View className="index">
      <Form className='form'>
        <Toast id="toast" />
        <Cell.Group inset>
          <Form.Item  >
            <Form.Label>话筒音量</Form.Label>
            <Form.Control>
              <Slider onChange={(e) => handleChange(e, 'volume')} value={ruleForm.volume} />
            </Form.Control>
          </Form.Item>
          <Form.Item  >
            <Form.Label>音乐音量</Form.Label>
            <Form.Control>
              <Slider onChange={(e) => handleChange(e, 'music')} value={ruleForm.music} />
            </Form.Control>
          </Form.Item>
          <Form.Item  >
            <Form.Label>音源设置</Form.Label>
            <Form.Control>
              <Radio.Group onChange={(e) => handleChange(e, 'source')} defaultValue="1" direction="horizontal" value={ruleForm.source} >
                <Radio name={0}>Line</Radio>
                <Radio name={1}>OPT</Radio>
                <Radio name={2}>ARC</Radio>
                <Radio name={3}>USB</Radio>
              </Radio.Group>
            </Form.Control>
          </Form.Item>
          <Form.Item  >
            <Form.Label>蓝牙开关</Form.Label>
            <Form.Control>
              <Switch size="24" onChange={(e) => handleChange(e, 'bleSwitch')} checked={ruleForm.bleSwitch} />
            </Form.Control>
          </Form.Item>
          <Form.Item  >
            <Form.Label>声卡开关</Form.Label>
            <Form.Control>
              <Switch size="24" onChange={(e) => handleChange(e, 'soundCadSwitch')} checked={ruleForm.soundCadSwitch} />
            </Form.Control>
          </Form.Item>
          <Form.Item  >
            <Form.Label>播放模式</Form.Label>
            <Form.Control>
              <Radio.Group onChange={(e) => handleChange(e, 'playMode')} defaultValue="1" direction="horizontal" value={ruleForm.playMode}>
                <Radio name={0}>列表循环</Radio>
                <Radio name={1}>随机播放</Radio>
                <Radio name={2}>单曲循环</Radio>
              </Radio.Group>
            </Form.Control>
          </Form.Item>
          <Form.Item  >
            <Form.Label>音乐控制</Form.Label>
            <Form.Control>
              <Radio.Group onChange={(e) => handleChange(e, 'musicControl')} defaultValue="1" direction="horizontal" value={ruleForm.musicControl}>
                <Radio name={0}>停止</Radio>
                <Radio name={1}>播放</Radio>
                <Radio name={2}>暂停</Radio>
                <Radio name={3}>上一曲</Radio>
                <Radio name={4}>下一曲</Radio>
              </Radio.Group>
            </Form.Control>
          </Form.Item>
          <Form.Item  >
            <Form.Label>超低音强度</Form.Label>
            <Form.Control>
              <Stepper onChange={(e) => handleChange(e, 'girth')} min={0} max={3} value={ruleForm.girth} />
            </Form.Control>
          </Form.Item>
          <Form.Item  >
            <Form.Label>混响模式</Form.Label>
            <Form.Control>
              <Stepper onChange={(e) => handleChange(e, 'reverberationMode')} min={1} max={6} value={ruleForm.reverberationMode} />
            </Form.Control>
          </Form.Item>
          <Form.Item  >
            <Form.Label>话筒混响</Form.Label>
            <Form.Control>
              <Stepper onChange={(e) => handleChange(e, 'microphoneReverberation')} min={0} max={32} value={ruleForm.microphoneReverberation} />
            </Form.Control>
          </Form.Item>
          <Form.Item  >
            <Form.Label>AI</Form.Label>
            <Form.Control>
              <Switch size="24" onChange={(e) => handleChange(e, 'AI')} checked={ruleForm.AI} />
            </Form.Control>
          </Form.Item>
        </Cell.Group>
      </Form>
    </View >
  )
}
