import { View, Image, Text } from '@tarojs/components'
import './index.less'
import { Switch, Slider } from "@taroify/core"
import ble from "@/assets/ble.png";
import soundCard from "@/assets/soundCard.png";
import game from "@/assets/game.png";
import ai from "@/assets/ai.png";
import Taro from '@tarojs/taro';
import { useCallback, useState } from 'react';
import { hex, strInsert } from '@/utils/sendOrder';
import { debounce } from '@/utils/index'
import BLESDK from '@/utils/ble'

export default function Index() {
  const sourceList = ['Line', 'OPT', 'ARC', 'USB']
  const [ruleForm, setRuleForm] = useState({
    volume: 0, //话筒音量
    music: 0, //音乐音量
    source: 0, //音源设置
    bleSwitch: 0, //蓝牙开关
    soundCadSwitch: 0, //声卡开关
    playMode: 0, //播放模式
    musicControl: 0, //音乐控制
    girth: 0, //超低音强度
    reverb: 1, //混响模式
    microphoneReverberation: 0, //话筒混响
    AI: 0, //AI
    playPause: 0, //播放暂停
  })
  const toPage = () => {
    Taro.navigateTo({ url: '/pages/game/index' })
  }
  const sendCode = useCallback(debounce((e) => {
    BLESDK.writeBleValue(new Uint8Array(strInsert(e)).buffer)
  }), [])
  const handleChange = (e, key) => {
    setRuleForm({
      ...ruleForm,
      [key]: e
    })
    let str = ''
    switch (key) {
      case 'volume':
        str = `7B05EA05B2${hex(e)}`
        break;
      case 'music':
        str = `7B05EA06B2${hex(e)}`
        break;
      case 'source':
        str = `7B05EA07B2${hex(e)}`
        break;
      case 'bleSwitch':
        str = `7B05EA08B2${hex(e)}`
        break;
      case 'soundCadSwitch':
        str = `7B05EA09B2${hex(e)}`
        break;
      case 'playMode':
        str = `7B05EA0AB2${hex(e)}`
        break;
      case 'musicControl':
        str = `7B05EA0BB2${hex(e)}`
        break;
      case 'girth':
        str = `7B05EA0CB2${hex(e)}`
        break;
      case 'reverb':
        str = `7B05EA0DB2${hex(e)}`
        break;
      case 'microphoneReverberation':
        str = `7B05EA0EB2${hex(e)}`
        break;
      case 'AI':
        str = `7B05EA11B2${hex(e)}`
        break;
    }
    sendCode(str)
  }
  // 音乐控制 0：停止 1：播放 2：暂停 3：上一曲 4：下一曲
  const musicControl = (num) => {
    let str = `7B05EA0BB2${hex(num)}`
    sendCode(str)
  }

  const control = (type) => {
    let num = ruleForm.girth
    if (type == "+" && ruleForm.girth < 3) {
      num++
    } else if (type == "-" && ruleForm.girth > 0) {
      num--
    }
    handleChange(num, 'girth')
  }

  return (
    <View className="index">
      <View className="headbox">
        <View className="source">
          <View className="title">音乐信号(音源)</View>
          <View className="list">
            {
              sourceList.map((item, index) => {
                return <View className={`list-item ${index == ruleForm.source ? 'active' : ''}`} key={index} onClick={() => handleChange(index, 'source')}>
                  <View className="yuan"></View>
                  <View className="line"></View>
                  <View className="text">{item}</View>
                </View>
              })
            }
          </View>
        </View>

        <View className="switch">
          <View className="title">开关状态</View>
          <View className="switch-item">
            <Image className='img' src={ble} />
            <Text className='text'>蓝牙</Text>
            <Switch size="20" onChange={(e) => handleChange(e, 'bleSwitch')} checked={ruleForm.bleSwitch} />
          </View>
          <View className="switch-item">
            <Image className='img' src={soundCard} />
            <Text className='text'>声卡</Text>
            <Switch size="20" onChange={(e) => handleChange(e, 'soundCadSwitch')} checked={ruleForm.soundCadSwitch} />
          </View>
        </View>
      </View>
      <View className='play'>
        <View className='play-icon pre' onClick={() => musicControl(3)}></View>
        <View className={ruleForm.playPause ? 'play-pause' : 'play-play'} onClick={() => {
          musicControl(ruleForm.playPause ? 2 : 1)
          handleChange(!ruleForm.playPause, 'playPause')
        }}></View>
        <View className='play-icon next' onClick={() => musicControl(4)}></View>
      </View>
      <View className='card'>
        <View className='funitem'>
          <View className='funitem-text'>音乐模式</View>
          <View className='mode'>
            <View className='icon l'></View>
            <View className='input'>MODE 1</View>
            <View className='icon r'></View>
          </View>
        </View>
      </View>
      <View className='card'>
        <View className='funitem'>
          <View className='funitem-text'>超低音强度</View>
          <View className='mode'>
            <View className='icon l' onClick={() => control("-")}></View>
            <View className='input'>{ruleForm.girth}</View>
            <View className='icon r' onClick={() => control("+")}></View>
          </View>
        </View>
      </View>
      <View className='card'>
        <View className='funitem'>
          <View className='funitem-text'>音乐音量</View>
          <View className='funitem-right'>
            <View className='num'>{ruleForm.volume}%</View>
            <Slider className="custom-color" min={0} max={100} size={4} onChange={(e) => handleChange(e, 'volume')} value={ruleForm.volume} >
              <Slider.Thumb>
                <View className="custom-thumb"></View>
              </Slider.Thumb>
            </Slider>
          </View>
        </View>
      </View>
      <View className='card'>
        <View className='funitem'>
          <View className='funitem-text'>话筒音量</View>
          <View className='funitem-right'>
            <View className='num'>{ruleForm.music}%</View>
            <Slider className="custom-color" min={0} max={100} size={4} onChange={(e) => handleChange(e, 'music')} value={ruleForm.music} >
              <Slider.Thumb>
                <View className="custom-thumb"></View>
              </Slider.Thumb>
            </Slider>
          </View>
        </View>
        <View className='funitem'>
          <View className='funitem-text'>话筒混响音量</View>
          <View className='funitem-right'>
            <View className='num'>{ruleForm.microphoneReverberation}%</View>
            <Slider className="custom-color" min={0} max={32} size={4} onChange={(e) => handleChange(e, 'microphoneReverberation')} value={ruleForm.microphoneReverberation} >
              <Slider.Thumb>
                <View className="custom-thumb"></View>
              </Slider.Thumb>
            </Slider>
          </View>
        </View>
        <View className='funitem'>
          <View className='funitem-text'>话筒混响模式</View>
          <View className='numlist'>
            {[1, 2, 3, 4, 5, 6].map(item => {
              return <View className={`numlist-num ${ruleForm.reverb == item ? 'active' : ''} `} key={item} onClick={() => handleChange(item, 'reverb')}>{item}</View>
            })}
          </View>
        </View>
      </View>
      <View className='btns'>
        <View className='btn-item' onClick={toPage}>
          <Image src={game} className='img' />
          GAME
        </View>
        <View className='btn-item'>
          <Image src={ai} className='img' />
          AI
        </View>
      </View>
    </View>
  )
}
